# The statement GOTO

[\[\<\<\<\]](ug_15.md) [\[\>\>\>\]](ug_17.md)

The statement `GOTO` is the most famous statement of all BASIC
languages. Many program theorists say that you should not ever use
`GOTO` and they may be right. Even so, the statement `GOTO` is part of
most programming languages and ScriptBasic is no exception.

Using the statement `GOTO` you can alter the execution order of the
statements. `GOTO` statements use labels and the labels can identify
program lines. The form of a `GOTO` statement is

    GOTO label

where the `label` is a label, which is defined somewhere in a program.

Labels should stand on the start of a line preceding a command. There
are two different label types in ScriptBasic. One is the conventional
BASIC style label, a decimal number. Any line can contain an unsigned
integer as a label at the start of the line. For example the following
code is perfectly legal in ScriptBasic:

    10 print "Hello word!!"
    20 REM
    30 print "This is line 30"

These types of line labels are available in ScriptBasic for
compatibility reasons, and there is no need to label each line in
ScriptBasic. The numeric labels can be present in any order and need not
increase. This is not compatible with old BASIC languages, but there is
little need for that. In those days these numbers were needed to help
the simple built-in editor to sort lines and to allow the programmer to
insert in new lines. (Did you program those days when the ZX80, ZX
Sinclair Spectrum, Atari 800XL, Commodore, Enterprise computers were on
top?) If you use these numeric labels the lines will be executed in the
order as they appear in the source file and not necessarily in
increasing label order.

The type of label, which is more modern and lets the programmer to write
more readable and as such easy to maintain programs are the alphanumeric
labels. These labels follow the syntax of any identifier of ScriptBasic,
stand on the start of the line and are followed by a colon. Both numeric
labels and alphanumeric labels can be used in `GOTO` statements.

``` 


10 print "hah"
line input a
if a = "n\n" then
  goto finish
end if
goto 10

finish:
 REM this is the end of the program
```

This program loops printing three characters and waiting for input after
each print so long as long the user types a single n character. When the
user types the n character the program executes the statement `GOTO` to
jump onto the label finish.

The labels are subject to name space modifications as any other variable
or user defined function names as described in the section [Name
spaces](ug_11.md).

All `GOTO` instructions should reference a local label. This means that
you can not jump out or jump into a function or subroutine code. You are
free to reuse label names once in each function or subroutine. In other
words if you use a label in a function you can use the same name as a
label without worry in another function or subroutine as well as you can
use it in global program code outside of all functions and subroutines
once. All these labels are different ones although all may have the same
name.

All the labels are local to the subroutine or function that the label is
defined in. This is implemented in ScriptBasic via name decoration. If
you declare a label named `MyLabel` on a line ScriptBasic automatically
converts it to `module::MyLabel'function`, where module is the name of
the actual module (this is `main` if no module is defined); `function`
is the name of the actual function or subroutine without the current
module name. If the label is in the global program code outside of all
functions and subroutines the function name is empty string. In this
case the label has the decorated form `module::MyLabel'`. The modules
can reference each-others labels using explicit module name notation.
However there is no possibility to reference any label, which is not in
the same function or subroutine.

-----

[\[\<\<\<\]](ug_15.md) [\[\>\>\>\]](ug_17.md)
